#--
# ucert
#
# A  Ruby library for enterprise user account certification / recertification / audit project
#
# Copyright (c)  CMBNY Risk Department
#++

require "rubyXL"


# Class to handle Citrix Sharefile user account
class Ucert::CitrixSharefileTracker
	include Ucert::Utils
	# Class constant variables

	attr_accessor  :verbose, :sharefile_user_status_report, :sharefile_user_group_entitlement_report, :file_sharefile_user_map
	attr_reader     :sharefile_2_ad_user, :sharefile_user_entitlement, :sharefile_user_group_entitlement

  # Instance default variables
  def initialize (params ={})
    @verbose=params.fetch(:verbose, false)
    # Citrix Sharefile user entitlement report generation instruction:
    # http://support.citrixonrow.com/en_US/ShareFile/all_files/SF070008
		# Please raise a new 'Application Security Request' in the Mantis ticketing system, in order to obtain
		# the latest copy
    @sharefile_user_entitlement_report = File.dirname(__FILE__)+"/../../data/citrix_sharefile/ShareFile_Access_Report.xlsx"
    # Sharefile  to AD user map file
    @file_sharefile_user_map =  File.dirname(__FILE__)+"/../../data/citrix_sharefile/sharefile_user_map.txt"
		# Load user map from the local cacsh file
		@sharefile_2_ad_user=load_known_user_map_from_file(@file_sharefile_user_map)
		# Load the user entitlement instance variable from the user report
		@sharefile_user_entitlement=parse_sharefile_user_entitlement_report(@sharefile_user_entitlement_report)
		# Procedure to add DN foreign key to the @sharefile_user_entitlement, by performing the AD search
		insert_dn
		# Save the user map to local cache file
		save!
	end

  # Parsing the Sharefile user entitlement report in text format
	def parse_sharefile_user_entitlement_report (file)
      begin
				puts "Start parsing Excel workbook file: #{file}" if @verbose
				sharefile_user_entitlement=Hash.new
				workbook = RubyXL::Parser.parse(file)
        worksheet = workbook[0]
				row_cnt=0
				user_index=String.new	# user record index
				header=Array.new
        worksheet.count.times  do |row|
					row_cnt+=1
					puts "Parsing workbook row: #{row_cnt}" if @verbose
					entry=Array.new
					# Processing Header Row
					if row_cnt==1
						0.upto(worksheet[row].size) do |col|
							if worksheet[row][col].nil?
								header.push(nil)
							else
								header.push(worksheet[row][col].value.to_s)
							end
						end
						next
					else
						0.upto(worksheet[row].size) do |col|
							if worksheet[row][col].nil?
								entry.push(nil)
							else
								entry.push(worksheet[row][col].value.to_s)
							end
						end
					end
          record = header.zip(entry)
					puts "User record: #{record}" if @verbose
					record_h = record.slice!(2,4).to_h
					puts "User record header: #{record_h}" if @verbose
					entitlement = record.to_h.reject {|k,v| k.nil?}
					puts "User entitlement: #{entitlement}" if @verbose
					next if entitlement["Path"].empty?
					user_index = record_h["Email"].upcase
          sharefile_user_entitlement[user_index] = record_h unless sharefile_user_entitlement.key?(user_index)
					sharefile_user_entitlement[user_index]["entitlements"]=Array.new unless sharefile_user_entitlement[user_index]["entitlements"]
					sharefile_user_entitlement[user_index]["entitlements"].push(entitlement)
      	end
        workbook=nil
        return sharefile_user_entitlement
      rescue => ee
				puts "Exception on method #{__method__}: #{ee}"
			end
	end

  # Retrieve the user index from the @sharefile_user_entitlement data structure
	def dn_2_index (dn)
			begin
        (1..@sharefile_user_entitlement.count).map do |index|
          return index if @sharefile_user_entitlement[index]["DN"]==dn
        end
			rescue => ee
				puts "Exception on method #{__method__}: #{ee}"
			end
	end

	# Procedures to add additonal field 'dn' into the @sharefile_user_entitlement data structure, by person the AD search
	def insert_dn
			begin
				tracker = Ucert::AdTracker.new(:verbose=>false)
				 @sharefile_user_entitlement.each do |index, record|
					puts "\n\nPerform DN lookup for record: #{record}" if @verbose
					key1 = index
					key2 = record['FirstName'] + ":" + record['LastName'] if record['FirstName'] && record['LastName']
					key3 = record['FirstName'] if record['FirstName']
					my_key = index.upcase
					puts "Perform 1st order search from the local cache: #{my_key}" if @verbose
					if @sharefile_2_ad_user.key?(my_key)
						dn=@sharefile_2_ad_user[my_key]
						# additional logic to update the existing DN record
						unless tracker.ad_person_records.key?(dn)
							dn = update_dn(tracker,dn)
						end
						puts "Found in the local cache file: #{dn}" if @verbose
					else
						if dn.nil? and !key1.nil?
							puts "Perform 2nd order search only if the 1st one fail, by using: #{key1}" if @verbose
							dn = tracker.ad_search_by_text(key1, "person")
						end
						if dn.nil? and !key2.nil?
							puts "Perform 3rd order search only if the last fail, by using: #{key2}" if @verbose
							dn = tracker.ad_search_by_text(key2, "person")
						end
						if dn.nil? and !key3.nil?
							puts "Perform 4th order search only if the last fail, by using: #{key3}" if @verbose
							dn = tracker.ad_search_by_text(key3, "person")
						end
					end
					@sharefile_user_entitlement[index]['DN'] = dn
				end
				tracker=nil
			rescue => ee
				puts "Exception on method #{__method__}: #{ee}"
			end
	end

	# Print out the user entitlement table in plain text, to be imported into database
	def print_user_entitlement
		begin
      puts "user Entitlement Report in Plain-text Format" if @verbose
			@sharefile_user_entitlement.first.each {|k,v| print k,"|"} if @verbose
			puts if @verbose
			@sharefile_user_entitlement.values.map do |rec|
				rec.each {|k,v| print v,"|"}
				puts
      end
		rescue => ee
			puts "Exception on method #{__method__}: #{ee}"
		end
	end
  alias_method :print_user, :print_user_entitlement


	# Save the Sharefile to AD user mapping relation into the cache file
	def save_sharefile_user_map!(file=@file_sharefile_user_map)
		puts "Saving the known ShareFile to AD user mapping relationship to file: #{file} ..." if @verbose
		begin
			timestamp=Time.now
			f=File.open(file, 'w')
			f.write "# ShareFile to AD user map file created by the #{self.class} class #{__method__} method at: #{timestamp}"
			@sharefile_user_entitlement.values.map do |record|
				key = record['Email'].upcase
				value = record['DN']
				f.write "\n#{key}|#{value}"
			end
			f.close
			puts "Sharefile to AD user map file is successfully saved to: #{file}" if @verbose
		rescue => ee
			puts "Exception on method #{__method__}: #{ee}" if @verbose
		end
	end
	alias_method  :save!, :save_sharefile_user_map!

	# Search user entitlement record by AD DN
	def db_search_by_dn (dn)
		begin
      puts "Perform search on the user entitlement records by AD DN: #{dn}" if @verbose
      @sharefile_user_entitlement.each do |key, val|
          return val if @sharefile_user_entitlement[key]['DN'].eql? dn
      end
			return nil
    rescue => ee
      puts "Exception on method #{__method__}: #{ee}"
    end
	end
	alias_method  :search_by_dn, :db_search_by_dn

	private :insert_dn, :parse_sharefile_user_entitlement_report, :save_sharefile_user_map!, :print_user_entitlement

end
